package com.nokia.finance.tasks.common.utils.psql;

import com.nokia.finance.tasks.common.exception.MyRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.exec.CommandLine;
import org.apache.commons.exec.DefaultExecutor;
import org.apache.commons.exec.ExecuteWatchdog;
import org.apache.commons.exec.PumpStreamHandler;
import org.springframework.util.StringUtils;

import java.io.ByteArrayOutputStream;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * psql命令工具
 */
@Slf4j
public class PsqlUtil {
    /**
     * 匹配psql copy成功结果
     */
    private static final Pattern PATTERN = Pattern.compile("^(COPY) (\\d+)$");

    /**
     * 导入csv
     *
     * @param script         脚本
     * @param dbHost         数据库主机
     * @param dbPort         数据库端口
     * @param dbUsername     数据库用户名
     * @param dbPassword     数据库密码
     * @param dbName         数据库名字
     * @param dbTable        数据库表
     * @param csv            csv
     * @param columns        字段
     * @param timeout        超时ms
     * @param minInsertCount 最小值插入数
     */
    public static void copyCsv(String script, String dbHost, String dbPort, String dbUsername, String dbPassword,
                               String dbName, String dbTable, String csv, String columns, Long timeout,
                               Long minInsertCount) {
        String command = "sh " + script;
        CommandLine commandLine = CommandLine.parse(command);
        commandLine.addArgument(dbHost);
        commandLine.addArgument(dbPort);
        commandLine.addArgument(dbUsername);
        commandLine.addArgument(dbPassword);
        commandLine.addArgument(dbName);
        commandLine.addArgument(dbTable);
        commandLine.addArgument(csv);
        commandLine.addArgument(columns);
        log.info("command: {}", commandLine);
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        ByteArrayOutputStream err = new ByteArrayOutputStream();
        DefaultExecutor executor = new DefaultExecutor();
        ExecuteWatchdog watchdog = new ExecuteWatchdog(timeout);
        executor.setWatchdog(watchdog);
        PumpStreamHandler streamHandler = new PumpStreamHandler(out, err);
        executor.setStreamHandler(streamHandler);
        try {
            int exitValue = executor.execute(commandLine);
            log.info("exitValue: {}", exitValue);
            String outString = out.toString();
            Long count = null;
            Matcher matcher = PATTERN.matcher(outString);
            if (matcher.find()) {
                count = Long.parseLong(matcher.group(2));
            }
            if (count == null) {
                throw new MyRuntimeException("导入数据失败");
            }
            log.info("插入 {} 条数据", count);
            if (minInsertCount != null && count < minInsertCount) {
                throw new MyRuntimeException(csv + " 数据异常，少于 " + minInsertCount);
            }
        } catch (Exception e) {
            if (watchdog.killedProcess()) {
                throw new MyRuntimeException("执行超时", e);
            }
            throw new MyRuntimeException(e);
        } finally {
            String outString = out.toString();
            String errString = err.toString();
            log.info("out: {}", outString);
            if (StringUtils.hasText(errString)) {
                log.error("err: {}", errString);
            }
        }
    }
}
