package com.nokia.finance.tasks.jobs.house.chengben;

import com.nokia.finance.tasks.common.exception.MyRuntimeException;
import com.nokia.finance.tasks.common.utils.psql.PsqlUtil;
import com.nokia.finance.tasks.config.JobConfig;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVParser;
import org.apache.commons.csv.CSVPrinter;
import org.apache.commons.csv.CSVRecord;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Stream;

/**
 * 河北成本管理系统自有类房产维修数据入库定时任务
 */
@Slf4j
@Service
public class HouseBuildingRepairMonthJob {
    private final JobConfig jobConfig;

    public HouseBuildingRepairMonthJob(JobConfig jobConfig) {
        this.jobConfig = jobConfig;
    }

    /**
     * 执行任务
     */
    @Scheduled(cron = "0 24 6 12 * ?")
    public void runJob() {
        // 数据目录
        Path dir = Paths.get(jobConfig.getHouseBuildingRepairMonthSourcePath());
        try (Stream<Path> stream = Files.list(dir)) {
            // 获取数据目录下的文件列表
            List<Path> pathList = stream.filter(t -> !t.toString().endsWith(".MD5")).sorted().toList();
            log.info("河北成本管理系统自有类房产维修数据文件列表: {}", pathList);
            if (CollectionUtils.isEmpty(pathList)) {
                throw new MyRuntimeException("河北成本管理系统自有类房产维修数据没有文件");
            }
            for (Path path : pathList) {
                singleJob(path);
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    /**
     * 处理单个文件
     *
     * @param path 文件路径
     */
    public void singleJob(Path path) throws Exception {
        List<Map<String, String>> list = readFile(path);
        List<Map<String, String>> distinctList = dataProcessing(path, list);
        Path csvPath = toCsv(path, distinctList);
        copyCsv(csvPath);
        move(path);
    }

    /**
     * 读取文件
     *
     * @param path 文件路径
     */
    public List<Map<String, String>> readFile(Path path) throws Exception {
        log.info("读取: {}", path);
        List<String> headers = Stream.of(
                "data_time_nlsd",
                "data_time", "form_type", "submitter", "submitter_dept_id", "form_no", "reimbursement_no",
                "start_time", "form_id", "investor", "fwxs_big_type", "jz_jzbh", "zrqk_jzmc", "location_info",
                "content_category", "estimate_cost", "describe", "del_flag", "final_cost", "description", "jz_jzid").toList();
        int headerSize = headers.size();
        char delimiter = 1;
        try (CSVParser parser = CSVFormat.DEFAULT.builder().setDelimiter(delimiter).build()
                .parse(new InputStreamReader(Files.newInputStream(path), StandardCharsets.UTF_8))
        ) {
            List<Map<String, String>> resultList = new ArrayList<>();
            for (CSVRecord csvRecord : parser) {
                Map<String, String> rowMap = new LinkedHashMap<>();
                for (int i = 0; i < headerSize; i++) {
                    String header = headers.get(i);
                    String value = csvRecord.get(i);
                    if (!"start_time".equals(header)) {
                        // 删除空白字符
                        value = StringUtils.trimAllWhitespace(value);
                    }
                    rowMap.put(header, value);
                }
                resultList.add(rowMap);
            }
            return resultList;
        }
    }

    /**
     * 数据加工
     *
     * @param path 文件路径
     * @param list 数据
     */
    public List<Map<String, String>> dataProcessing(Path path, List<Map<String, String>> list) {
        for (Map<String, String> map : list) {
            String startTime = map.get("start_time");
            LocalDateTime localDateTime = LocalDateTime.parse(startTime,
                    DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            String yearMonth = localDateTime.format(DateTimeFormatter.ofPattern("yyyyMM"));
            String year = String.valueOf(localDateTime.getYear());
            String month = String.valueOf(localDateTime.getMonthValue());
            map.put("year_month", yearMonth);
            map.put("year_no", year);
            map.put("month_no", month);
            String formType = map.get("form_type");
            String repairType = "";
            if ("13".equals(formType)) {
                repairType = "零星维修";
            } else if ("14".equals(formType)) {
                repairType = "应急维修";
            } else if ("15".equals(formType) || "16".equals(formType)) {
                repairType = "项目制维修";
            }
            map.put("repair_type", repairType);
            map.put("source", path.getFileName().toString());
        }
        return list;
    }

    /**
     * 生成csv
     *
     * @param path 源文件路径
     * @param list 数据
     */
    public Path toCsv(Path path, List<Map<String, String>> list) throws Exception {
        log.info("条数：{}", list.size());
        Files.createDirectories(Paths.get(jobConfig.getHouseBuildingRepairMonthHistoryPath()));
        Path csvPath = Paths.get(jobConfig.getHouseBuildingRepairMonthHistoryPath() + path.getFileName() + ".csv");
        try (OutputStreamWriter osw = new OutputStreamWriter(Files.newOutputStream(csvPath),
                StandardCharsets.UTF_8);
             CSVPrinter printer = new CSVPrinter(osw, CSVFormat.DEFAULT)) {
            // 添加bom头避免excel乱码
            osw.write('\ufeff');
            Map<String, String> header = list.get(0);
            // 表头
            printer.printRecord(header.keySet());
            for (Map<String, String> map : list) {
                printer.printRecord(map.values());
            }
        }
        return csvPath;
    }

    /**
     * 导入数据库
     *
     * @param path 文件路径
     */
    public void copyCsv(Path path) {
        String dbTable = "house.building_repair_month";
        String csv = path.toString();
        String columns = "(data_time_nlsd,data_time,form_type,submitter,submitter_dept_id,form_no,reimbursement_no,start_time,form_id,investor,fwxs_big_type,jz_jzbh,zrqk_jzmc,location_info,content_category,estimate_cost,describe,del_flag,final_cost,description,jz_jzid,year_month,year_no,month_no,repair_type,source)";
        Long timeout = 60000L;
        PsqlUtil.copyCsv(jobConfig.getCopyScriptPath(), jobConfig.getDbHost(), jobConfig.getDbPort(),
                jobConfig.getDbUsername(), jobConfig.getDbPassword(), jobConfig.getDbName(), dbTable, csv, columns,
                timeout, null);
    }

    /**
     * 移动源文件到历史文件夹
     *
     * @param path 源文件路径
     */
    public void move(Path path) throws Exception {
        Files.move(Paths.get(path + ".MD5"),
                Paths.get(jobConfig.getHouseBuildingRepairMonthHistoryPath(), path.getFileName().toString() + ".MD5"),
                StandardCopyOption.REPLACE_EXISTING);
        Files.move(path, Paths.get(jobConfig.getHouseBuildingRepairMonthHistoryPath(), path.getFileName().toString()),
                StandardCopyOption.REPLACE_EXISTING);
    }
}
